describe("LocalStorageEngine", function() {

	var localStorageEngine;
	var dataSourceName = "/registerCountries";
	var dataSourceNameComplexKey = "/registerCountries_complex";
	var requestFactory = {};
	var requestEngine = {};
	var zhPromise = new ZHPromise(Q);
	var Query = new QueryBuilder();
	var thenable = function(data, name) {
		return {
			then: function(fn) {
				if (fn) {
					return fn(data, name);
				} else {
					return thenable(data, name);
				}
			}
		};
	};
	var requestDataSourceEngine = {
		getDataSource: function(name, callBack, filter, page, itemsPerPage, errorCallBack) {
			var data = localStorageEngine.processFilter(ZhHelper.generateData(1000), filter);
			if (callBack) {
				callBack(data, dataSourceName);
			}
			return thenable(data, dataSourceName);
		},
		setDataSource: function(name, data, callBack, retryCount, timeout, retryCallback) {
			data.forEach(function(item) {
				item.setted = true;
			});
			data = {
				status: 200,
				data: data
			};
			if (callBack) {
				callBack(data);
			}
			return thenable(data);
		}
	};
	var defered = Q.defer();
	defered.resolve({});
	var templateManager = {
		getDataSources: function() {
			return defered.promise;
		},
		project: {
			databaseName: "indexedDBTestt"
		},
		container: {
			name: 'container',
			getActiveWidget: function() {
				return {
					name: 'widget'
				};
			}
		}
	};
	var ALL_RECORDS = 999999999;
	var repository = {
		"registerCountries": {
			"localKey": ["id"]
		}
	};
	var repositoryComplex = {
		"registerCountries_complex": {
			"localKey": ["id", "name"]
		}
	};
	var dataSourceHolder = new DataSourceHolder(zhPromise, {
		requestMetaData: function(filename, data, callBack) {
			var repositoryToReturn = filename.indexOf(dataSourceNameComplexKey) != -1  ? repositoryComplex : repository;
			if (callBack) {
				callBack(repositoryToReturn);
			}
			var promise = {
				then: function(fn) {
					if (fn) {
						return fn(repositoryToReturn);
					} else {
						return promise;
					}
				}
			};
			return promise;
		}
	}, console);
	var dataToFilterTest = [{
			"number": 1,
			"name": "Mateus Freira dos Santos"
	}, {
			"number": 15,
			"name": "Airton cabral Neto"
	}, {
			"number": 24,
			"name": "Rhaylander Almeida"
	}, {
			"number": 3,
			"name": "Zed Martins"
	},
		{
			"number": 4,
			"name": null
	}];

	beforeEach(inject(function(_$filter_) {
		var zhUtil = new ZhUtil({}, Query, _$filter_);
		localStorageEngine = new LocalStorageEngine(console, requestDataSourceEngine, requestFactory,
			requestEngine, templateManager, dataSourceHolder, zhPromise, zhUtil, Query);
	}));

	it("Filter 'in' should filter using string in(x,y)", function() {

		var items = localStorageEngine.inFilter(dataToFilterTest, {
			name: "number",
			value: "1,15"
		});
		expect(items.length).toBe(2);
		expect(items[1].number).toBe(15);
	});

	it("Filter 'in' should filter using array in(x,y)", function() {
		var items = localStorageEngine.inFilter(dataToFilterTest, {
			name: "number",
			value: [1, 24]
		});
		expect(items.length).toBe(2);
		expect(items[1].number).toBe(24);
	});

	it("Filter 'not in' should filter using string in(x,y)", function() {
		var items = localStorageEngine.notInFilter(dataToFilterTest, {
			name: "number",
			value: '1,24,4'
		});
		expect(items.length).toBe(2);
		expect(items[1].number).toBe(3);
	});

	it("Filter 'not in' should filter using array in(x,y)", function() {
		var items = localStorageEngine.notInFilter(dataToFilterTest, {
			name: "number",
			value: [1, 24, 4]
		});
		expect(items.length).toBe(2);
		expect(items[1].number).toBe(3);
	});

	it("Filter 'like': starting values in string", function() {
		var items = localStorageEngine.likeFilter(dataToFilterTest, {
			name: "name",
			value: 'Zed%'
		});
		expect(items.length).toBe(1);
		expect(items[0].number).toBe(3);
	});

	it("Filter 'like': values string on end", function() {
		var items = localStorageEngine.likeFilter(dataToFilterTest, {
			name: "name",
			value: '%Neto'
		});
		expect(items.length).toBe(1);
		expect(items[0].number).toBe(15);
	});

	it("Filter 'like': any place of the string", function() {
		var items = localStorageEngine.likeFilter(dataToFilterTest, {
			name: "name",
			value: '%a%'
		});
		expect(items.length).toBe(4);
	});

	it("Filter 'not like': starting values in string", function() {
		var items = localStorageEngine.notLikeFilter(dataToFilterTest, {
			name: "name",
			value: 'Zed%'
		});
		expect(items.length).toBe(4);
		expect(items[2].number).toBe(24);
	});

	it("Filter 'not like': values string on end", function() {
		var items = localStorageEngine.notLikeFilter(dataToFilterTest, {
			name: "name",
			value: '%Neto'
		});
		expect(items.length).toBe(4);
		expect(items[1].number).toBe(24);
	});

	it("Filter 'not like': any place of the string", function() {
		var items = localStorageEngine.notLikeFilter(dataToFilterTest, {
			name: "name",
			value: '%d%'
		});
		expect(items.length).toBe(2);
		expect(items[0].number).toBe(15);
	});

	it("Filter 'like all': starting values in string", function() {
		var items = localStorageEngine.likeAllFilter(dataToFilterTest, {
			value: 'Zed%',
			name: "*"
		});
		expect(items.length).toBe(1);
		expect(items[0].number).toBe(3);
	});

	it("Filter 'like all': values string on end", function() {
		var items = localStorageEngine.likeAllFilter(dataToFilterTest, {
			value: '%Neto',
			name: "*"
		});
		expect(items.length).toBe(1);
		expect(items[0].number).toBe(15);
	});

	it("Filter 'like all': any place of the string", function() {
		var items = localStorageEngine.likeAllFilter(dataToFilterTest, {
			value: '%a%',
			name: "*"
		});
		expect(items.length).toBe(4);
	});

	it("Filter 'equalFilter' must find one register with one determined value", function() {
		var items = localStorageEngine.equalFilter(dataToFilterTest, {
			name: "name",
			value: 'Rhaylander Almeida'
		});
		expect(items.length).toBe(1);
		expect(items[0].number).toBe(24);
	});

	it("Filter 'notEqualFilter' must find all registers with a diferent value", function() {
		var items = localStorageEngine.notEqualFilter(dataToFilterTest, {
			name: "name",
			value: 'Rhaylander Almeida'
		});
		expect(items.length).toBe(4);
		expect(items[2].number).toBe(3);
	});

	it("Filter 'lessFilter' with string field: must return all items less than 'Rhaylander Almeida'",
		function() {
			var items = localStorageEngine.lessFilter(dataToFilterTest, {
				name: "name",
				value: 'Rhaylander Almeida'
			});
			expect(items.length).toBe(2);
		});

	it("Filter 'lessFilter' with number field: must return all items less than 24", function() {
		var items = localStorageEngine.lessFilter(dataToFilterTest, {
			name: "number",
			value: 24
		});
		expect(items.length).toBe(4);
		expect(items[2].number).toBe(3);
	});

	it("Filter 'lessEqualFilter' with string field: must return all items less or equal than 'Rhaylander Almeida'", function() {
		var items = localStorageEngine.lessEqualFilter(dataToFilterTest, {
			name: "name",
			value: 'Rhaylander Almeida'
		});
		expect(items.length).toBe(3);
		expect(items[2].number).toBe(24);
	});

	it("Filter 'lessEqualFilter' with number field: must return all items less or equal than 24",
		function() {
			var items = localStorageEngine.lessEqualFilter(dataToFilterTest, {
				name: "number",
				value: 24
			});
			expect(items.length).toBe(5);
			expect(items[2].number).toBe(24);
		});

	it("Filter 'moreFilter' with string field: must return all items more than 'Rhaylander Almeida'",
		function() {
			var items = localStorageEngine.moreFilter(dataToFilterTest, {
				name: "name",
				value: 'Rhaylander Almeida'
			});
			expect(items.length).toBe(1);
		});

	it("Filter 'moreFilter' with number field: must return all items more than 24", function() {
		var items = localStorageEngine.moreFilter(dataToFilterTest, {
			name: "number",
			value: 24
		});
		expect(items.length).toBe(0);
	});

	it("Filter 'moreEqualFilter' with string field: must return all items more or equal than 'Rhaylander Almeida'",function() {
		var items = localStorageEngine.moreEqualFilter(dataToFilterTest, {
			name: "name",
			value: 'Rhaylander Almeida'
		});
		expect(items.length).toBe(2);
		expect(items[1].number).toBe(3);
	});

	it("Filter 'moreEqualFilter' with number field: must return all items more or equal than 24", function() {
		var items = localStorageEngine.moreEqualFilter(dataToFilterTest, {
			name: "number",
			value: 24
		});
		expect(items.length).toBe(1);
		expect(items[0].number).toBe(24);
	});

	it("The method 'queryFilter' must choose the correct filter and run it", function() {
		var items = localStorageEngine.queryFilter(dataToFilterTest, {
			name: "number",
			value: 24,
			"operator": "="
		});
		expect(items.length).toBe(1);
		expect(items[0].number).toBe(24);
		expect(items[0].name).toBe("Rhaylander Almeida");
	});

	it("Filter 'isNullFilter'", function() {
		var items = localStorageEngine.isNullFilter(dataToFilterTest, {
			name: "name"
		});
		expect(items.length).toBe(1);
		expect(items[0].number).toBe(4);
		expect(items[0].name).toBe(null);
	});

	it("Filter 'isNotNullFilter'", function() {
		var items = localStorageEngine.isNotNullFilter(dataToFilterTest, {
			name: "name"
		});
		expect(items.length).toBe(4);
		expect(items[0].number).toBe(1);
		expect(items[0].name).toBe("Mateus Freira dos Santos");
		expect(items[1].name).toBe("Airton cabral Neto");
		expect(items[2].name).toBe("Rhaylander Almeida");
		expect(items[3].name).toBe("Zed Martins");
	});

	it("The method 'processFilter' must run all filters and return the filtered value", function() {
		var filters = [{
			name: "number",
			value: [1, 15, 24],
			"operator": "IN"
		}, {
			name: "name",
			value: '%an%',
			"operator": "LIKE"
		}, {
			name: "number",
			value: 24,
			"operator": "="
		}];
		var items = localStorageEngine.processFilter(dataToFilterTest, filters);
		expect(items.length).toBe(1);
		expect(items[0].number).toBe(24);
		expect(items[0].name).toBe("Rhaylander Almeida");
	});

	it("The method 'Save' must save the data with promises", function(done) {
		var processed = false;
		var dataToSave = [];
		for (var i = 0; i < 50; i++) {
			dataToSave.push({
				"id": i,
				"name": 'Afghanistan ' + i
			});
		}
		var promises = [];
		var startTime;
		localStorageEngine.remove(dataSourceName, []).then(function() {
			startTime = new Date().getTime();
			dataToSave.forEach(function(data) {
				promises.push(localStorageEngine.save(dataSourceName, data));
			});
			return zhPromise.all(promises);
		}).then(function() {
			var endTime = new Date().getTime();
			var expendTime = (endTime - startTime);
			console.log("The method 'Save' must save the data with promises: " + expendTime + "ms");
			expect(expendTime).toBeLessThan(7000);
			return localStorageEngine.getDataSource(dataSourceName);
		}).then(function(result) {
			expect(result.length).toEqual(50);
			processed = true;
			done();
		});
	});

	it("The method 'Save' must save with complex localKeys", function(done) {
		var processed = false;
		var dataToSave = [];
		for (var i = 0; i < 50; i++) {
			dataToSave.push({
				"id": 1,
				"name": 'Afghanistan '+i
			});
		}
		//Add a new weird data in the dataSource
		dataToSave.push({
			"id": 1,
			"name": 'Afghanistan 1',
			"update": true,
		});
		var promises = [];
		var startTime;
		localStorageEngine.remove(dataSourceNameComplexKey, []).then(function() {
			startTime = new Date().getTime();
			dataToSave.forEach(function(data) {
				promises.push(localStorageEngine.save(dataSourceNameComplexKey, data));
			});
			return zhPromise.all(promises);
		}).then(function() {
			var endTime = new Date().getTime();
			var expendTime = (endTime - startTime);
			console.log("The method 'Save' must save the data with promises: " + expendTime + "ms");
			expect(expendTime).toBeLessThan(7000);
			return localStorageEngine.getDataSource(dataSourceNameComplexKey);
		}).then(function(result) {
			expect(result.length).toEqual(50);
			processed = true;
			done();
		});
	});

	it("The method 'Save' must save the data", function(done) {
		var processed = false;
		var startTime = new Date().getTime();
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("The method 'Save' must save the data: " + ((new Date().getTime()) - startTime) +
				"ms");
			expect(data.length).toEqual(1001);
			processed = true;
			done();
		});


	});

	it("The method 'Save' must merge and save the data", function(done) {
		var processed = false;
		var startTime;
		var expendTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			var dataToSave = [];
			for (var i = 10; i < 30; i++) {
				dataToSave.push({
					"id": i,
					"name": 'Afghanistan ' + (i * 1000),
					"propA": "Afghanistan " + (i * 1000)
				});
			}
			startTime = new Date().getTime();
			return localStorageEngine.save(dataSourceName, dataToSave);
		}).then(function() {
			expendTime = (new Date().getTime()) - startTime;
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("The method 'Save' must merge and save the data: " + expendTime + "ms");
			expect(data.length).toEqual(1001);
			var nameToFind = 'Afghanistan 10000';
			var items = data.filter(function(d) {
				return d.id === 10;
			});
			expect(items.length).toEqual(1);
			expect(items[0].name).toEqual(nameToFind);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': must get all data", function(done) {
		var processed = false;
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(1001);
			}, [], 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': must get all data: " + ((new Date().getTime()) -
				startTime) + "ms");
			expect(data.length).toEqual(1001);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': order by", function(done) {
		var processed = false;
		var filters = Query.build().where('id').lower(10).orderBy('id', Query.ORDER_DESC).where();
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(10);
			}, filters);
		}).then(function(data) {
			console.log("Method 'getDataSource': order by: " + ((new Date().getTime()) - startTime) +
				"ms");
			expect(data.length).toEqual(10);
			var inc = 9;
			data.forEach(function(item) {
				expect(item.id).toEqual(inc);
				inc--;
			});
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter '>'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 650,
			"operator": ">"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(350);
			}, filters);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter '>': " + ((new Date().getTime()) - startTime) +
				"ms");
			expect(data.length).toEqual(350);
			expect(data[0].id).toEqual(651);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter '>='", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 650,
			"operator": ">="
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(351);
			}, filters);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter '>=': " + ((new Date().getTime()) - startTime) +
				"ms");
			expect(data.length).toEqual(351);
			expect(data[0].id).toEqual(650);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter '<'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 650,
			"operator": "<"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(650);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter '<': " + ((new Date().getTime()) - startTime) +
				"ms");
			expect(data.length).toEqual(650);
			expect(data[data.length - 1].id).toEqual(649);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter '<='", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 650,
			"operator": "<="
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(651);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter '<=': " + ((new Date().getTime()) - startTime) +
				"ms");
			expect(data.length).toEqual(651);
			expect(data[data.length - 1].id).toEqual(650);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter 'BETWEEN'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: [201, 500],
			"operator": "BETWEEN"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(300);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter 'BETWEEN': " + ((new Date().getTime()) -
				startTime) + "ms");
			expect(data.length).toEqual(300);
			expect(data[0].id).toEqual(201);
			expect(data[data.length - 1].id).toEqual(500);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter 'NOT_BETWEEN'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: [201, 500],
			"operator": "NOT_BETWEEN"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(701);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter 'NOT_BETWEEN': " + ((new Date().getTime()) -
				startTime) + "ms");
			expect(data.length).toEqual(701);
			expect(data[0].id).toEqual(0);
			expect(data[data.length - 1].id).toEqual(1000);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter '='", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 501,
			"operator": "="
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(1);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter '=': " + ((new Date().getTime()) - startTime) +
				"ms");
			expect(data.length).toEqual(1);
			expect(data[0].id).toEqual(501);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter '<>'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 501,
			"operator": "<>"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(1000);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter '<>': " + ((new Date().getTime()) - startTime) +
				"ms");
			expect(data.length).toEqual(1000);
			expect(data[501].id).toEqual(502);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter 'IN'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: [2, 60, 99, 250, 600],
			"operator": "IN"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(5);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter 'IN': " + ((new Date().getTime()) - startTime) +
				"ms");
			expect(data.length).toEqual(5);
			expect(data[0].id).toEqual(2);
			expect(data[data.length - 1].id).toEqual(600);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter 'NOT_IN'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: [2, 60, 99, 250, 600],
			"operator": "NOT_IN"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(996);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter 'NOT_IN': " + ((new Date().getTime()) -
				startTime) + "ms");
			expect(data.length).toEqual(996);
			expect(data[0].id).toEqual(0);
			expect(data[2].id).toEqual(3);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter 'LIKE'", function(done) {
		var processed = false;
		var filters = [{
			name: "name",
			value: "Afghanistan 60%",
			"operator": "LIKE"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(11);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter 'LIKE': " + ((new Date().getTime()) -
				startTime) + "ms");
			expect(data.length).toEqual(11);
			expect(data[0].id).toEqual(60);
			expect(data[data.length - 1].id).toEqual(609);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter 'NOT_LIKE'", function(done) {
		var processed = false;
		var filters = [{
			name: "name",
			value: "Afghanistan 60%",
			"operator": "NOT_LIKE"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(990);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter 'NOT_LIKE': " + ((new Date().getTime()) -
				startTime) + "ms");
			expect(data.length).toEqual(990);
			data = data.sort(function(a, b) {
				return a.id - b.id;
			});
			expect(data[60].id).toEqual(61);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter 'LIKE_ALL'", function(done) {
		var processed = false;
		var filters = [{
			"name": "*",
			value: "Afghanistan 60%",
			"operator": "LIKE_ALL"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(22);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter 'LIKE_ALL': " + ((new Date().getTime()) -
				startTime) + "ms");
			expect(data.length).toEqual(22);
			expect(data[0].id).toEqual(50);
			expect(data[1].id).toEqual(60);
			expect(data[data.length - 11].id).toEqual(599);
			expect(data[data.length - 1].id).toEqual(609);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter IS_NULL", function(done) {
		var processed = false;
		var filters = [{
			name: "propA",
			operator: "IS_NULL"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(1);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter 'IS_NULL': " + ((new Date().getTime()) -
				startTime) + "ms");
			expect(data.length).toEqual(1);
			expect(data[0].id).toEqual(1000);
			expect(data[0].propA).toEqual(null);
			expect(data[data.length - 1].id).toEqual(1000);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': filter IS_NOT_NULL", function(done) {
		var processed = false;
		var filters = [{
			name: "propA",
			operator: "IS_NOT_NULL"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(1000);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log("Method 'getDataSource': filter 'IS_NOT_NULL': " + ((new Date().getTime()) -
				startTime) + "ms");
			expect(data.length).toEqual(1000);
			expect(data[0].id).toEqual(0);
			expect(data[data.length - 1].id).toEqual(999);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': apply multiple filters", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: [500, 699],
			"operator": "BETWEEN"
		}, {
			name: "name",
			value: "Afghanistan 55%",
			"operator": "LIKE"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(10);
			}, filters, 1, ALL_RECORDS);
		}).then(function(data) {
			console.log(
				"Method 'getDataSource': must get all data, apply filters and get the first page: " + ((
					new Date().getTime()) - startTime) + "ms");
			expect(data.length).toEqual(10);
			expect(data[0].id).toEqual(550);
			expect(data[data.length - 1].id).toEqual(559);
			processed = true;
			done();
		});


	});

	it("Method 'getDataSource': must get all data, apply filters and get the first page", function(
		done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: [500, 699],
			"operator": "BETWEEN"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.getDataSource(dataSourceName, function(data) {
				expect(data.length).toEqual(30);
			}, filters, 1, 30);
		}).then(function(data) {
			console.log(
				"Method 'getDataSource': must get all data, apply filters and get the first page: " + ((
					new Date().getTime()) - startTime) + "ms");
			expect(data.length).toEqual(30);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter '>'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 650,
			"operator": ">"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, undefined, undefined, undefined,
				ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter '>': " + ((new Date().getTime()) - startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(651);
			expect(data[0].id).toEqual(0);
			expect(data[data.length - 1].id).toEqual(650);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter '>='", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 650,
			"operator": ">="
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter '>=': " + ((new Date().getTime()) - startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(650);
			expect(data[0].id).toEqual(0);
			expect(data[data.length - 1].id).toEqual(649);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter '<'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 650,
			"operator": "<"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter '<': " + ((new Date().getTime()) - startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(351);
			expect(data[0].id).toEqual(650);
			expect(data[data.length - 1].id).toEqual(1000);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter '<='", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 650,
			"operator": "<="
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter '<=': " + ((new Date().getTime()) - startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(350);
			expect(data[0].id).toEqual(651);
			expect(data[data.length - 1].id).toEqual(1000);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter 'BETWEEN'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: [100, 500],
			"operator": "BETWEEN"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter 'BETWEEN': " + ((new Date().getTime()) - startTime) +
				"ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(600);
			expect(data[0].id).toEqual(0);
			expect(data[data.length - 1].id).toEqual(1000);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter 'NOT_BETWEEN'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: [100, 500],
			"operator": "NOT_BETWEEN"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter 'NOT_BETWEEN': " + ((new Date().getTime()) -
				startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(401);
			expect(data[0].id).toEqual(100);
			expect(data[data.length - 1].id).toEqual(500);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter '='", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 650,
			"operator": "="
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter '=': " + ((new Date().getTime()) - startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1000);
			expect(data[650].id).toEqual(651);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter '<>'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: 650,
			"operator": "<>"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter '<>': " + ((new Date().getTime()) - startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1);
			expect(data[0].id).toEqual(650);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter 'IN'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: [6, 10, 69, 105, 500, 999],
			"operator": "IN"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter 'IN': " + ((new Date().getTime()) - startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(995);
			expect(data[6].id).toEqual(7);
			expect(data[data.length - 1].id).toEqual(1000);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter 'NOT_IN'", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: [6, 10, 69, 105, 500, 999],
			"operator": "NOT_IN"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter 'NOT_IN': " + ((new Date().getTime()) - startTime) +
				"ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(6);
			expect(data[0].id).toEqual(6);
			expect(data[data.length - 1].id).toEqual(999);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter 'LIKE'", function(done) {
		var processed = false;
		var filters = [{
			name: "name",
			value: 'Afghanistan 50%',
			"operator": "LIKE"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter 'LIKE': " + ((new Date().getTime()) - startTime) +
				"ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(990);
			expect(data[50].id).toEqual(51);
			expect(data[499].id).toEqual(510);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter 'NOT_LIKE'", function(done) {
		var processed = false;
		var filters = [{
			name: "name",
			value: 'Afghanistan 50%',
			"operator": "NOT_LIKE"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter 'NOT_LIKE': " + ((new Date().getTime()) - startTime) +
				"ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(11);
			expect(data[0].id).toEqual(50);
			expect(data[data.length - 1].id).toEqual(509);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter 'LIKE_ALL'", function(done) {
		var processed = false;
		var filters = [{
			name: "*",
			value: 'Afghanistan 50%',
			"operator": "LIKE_ALL"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter 'LIKE_ALL': " + ((new Date().getTime()) - startTime) +
				"ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(979);
			expect(data[40].id).toEqual(41);
			expect(data[49].id).toEqual(51);
			expect(data[488].id).toEqual(510);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter 'IS_NULL'", function(done) {
		var processed = false;
		var filters = [{
			name: "propA",
			operator: "IS_NULL"
		}];
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter 'IS_NULL': " + ((new Date().getTime()) - startTime) +
				"ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1000);
			expect(data[50].id).toEqual(50);
			expect(data[499].id).toEqual(499);
			processed = true;
			done();
		});


	});

	it("Method 'remove': filter 'IS_NOT_NULL'", function(done) {
		var processed = false;
		var filters = [{
			name: "propA",
			operator: "IS_NOT_NULL"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': filter 'IS_NOT_NULL': " + ((new Date().getTime()) -
				startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1);
			expect(data[0].id).toEqual(1000);
			expect(data[0].propA).toEqual(null);
			expect(data[data.length - 1].id).toEqual(1000);
			processed = true;
			done();
		});


	});

	it("Method 'remove': apply multiple filters", function(done) {
		var processed = false;
		var filters = [{
			name: "id",
			value: [500, 699],
			"operator": "BETWEEN"
		}, {
			name: "name",
			value: "Afghanistan 55%",
			"operator": "LIKE"
		}];
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, filters);
		}).then(function() {
			console.log("Method 'remove': apply multiple filters: " + ((new Date().getTime()) -
				startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(991);
			expect(data[550].id).toEqual(560);
			processed = true;
			done();
		});


	});

	it("Method 'remove': must remove all data from dataSource", function(done) {
		var processed = false;
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, []);
		}).then(function() {
			console.log("Method 'remove': must remove all data from dataSource: " + ((new Date().getTime()) -
				startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName);
		}).then(function(data) {
			expect(data.length).toEqual(0);
			processed = true;
			done();
		});


	});

	it("Method 'remove': by dataSource", function(done) {
		var processed = false;
		var dataSource = [{ "id": 0, "name": 'Afghanistan 0', "propA": "Afghanistan 10" }, { "id": 20, "name": 'Afghanistan 20', "propA": "Afghanistan 30" }];
		var hasCallCallback = false;
		var startTime;

		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.remove(dataSourceName, dataSource, function(){ hasCallCallback = true; });
		}).then(function() {
			console.log("Method 'remove': by dataSource: " + ((new Date().getTime()) - startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(999);
			expect(data[0].id).toEqual(1);
			expect(data[17].id).toEqual(18);
			expect(data[18].id).toEqual(19);
			expect(data[19].id).toEqual(21);
			expect(data[20].id).toEqual(22);
			expect(hasCallCallback).toEqual(true);
			processed = true;
			done();
		});


	});

	it("Method 'setDataSource' must change the data from dataSource", function(done) {
		var processed = false;
		var dataToSet = [];
		for (var i = 0; i < 50; i++) {
			dataToSet.push({
				"id": i,
				"name": 'Afghanistan ' + i + " setted"
			});
		}
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(1001);
			startTime = new Date().getTime();
			return localStorageEngine.setDataSource(dataSourceName, dataToSet, function(data) {
				expect(data.length).toEqual(50);
				expect(data[0].name).toBe('Afghanistan 0 setted');
			});
		}).then(function() {
			console.log("Method 'setDataSource' must change the data from dataSource: " + ((new Date()
				.getTime()) - startTime) + "ms");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(50);
			expect(data[0].name).toBe('Afghanistan 0 setted');
			processed = true;
			done();
		});


	});

	it("Method 'setLocalVar' must save one var and 'getLocalVar' must get one var", function() {
		var data = {
			name: "Rhaylander Almeida"
		};
		localStorageEngine.setLocalVar("local_var", data);
		var dataReturned = localStorageEngine.getLocalVar("local_var");
		expect(dataReturned.name).toBe(data.name);
	});

	it("Method 'download' must get the data from server and update the local data", function(done) {
		var filters = [{
			name: "id",
			value: [500, 699],
			"operator": "BETWEEN"
		}];
		var dataSource = [];
		for (var i = 600; i < 750; i++) {
			dataSource.push({
				"id": i,
				"name": 'Afghanistan ' + i,
				'added': 'manually'
			});
		}
		var startTime;
		ZhHelper.clearDataSource(localStorageEngine, dataSourceName).then(function() {
			return localStorageEngine.save(dataSourceName, dataSource);
		}).then(function() {
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(150);
			expect(data[0].name).toEqual("Afghanistan 600");
			startTime = new Date().getTime();
			return localStorageEngine.download(dataSourceName, filters);
		}).then(function(data) {
			console.log("Method 'download' must get the data from server and update the local data: " +
				((new Date().getTime()) - startTime) + "ms");
			expect(data.length).toEqual(200);
			expect(data[0].id).toEqual(500);
			expect(data[0].name).toEqual("Afghanistan 500");
			return localStorageEngine.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}).then(function(data) {
			expect(data.length).toEqual(250);
			expect(data[0].name).toEqual("Afghanistan 500");
			expect(data[data.length - 1].name).toEqual("Afghanistan 749");
			expect(data[data.length - 1].added).toEqual("manually");
			done();
		});


	});

	it("Method 'saveInServer' must save the local data in server", function(done) {
		var processed = false;
		var startTime;
		ZhHelper.reloadDataSource(localStorageEngine, dataSourceName).then(function() {
			startTime = new Date().getTime();
			return localStorageEngine.saveInServer(dataSourceName, function(data) {
				expect(data.status).toEqual(200);
				expect(data.data.length).toEqual(1001);
				data.data.forEach(function(item) {
					expect(item.setted).toEqual(true);
				});
			});
		}).then(function(data) {
			console.log("Method 'saveInServer' must save the local data in server: " + ((new Date().getTime()) -
				startTime) + "ms");
			expect(data.status).toEqual(200);
			expect(data.data.length).toEqual(1001);
			processed = true;
			done();
		});


	});

});