describe("MasterDetailController", function () {
	var masterDetailController;
	var $scope;
	var $rootScope;
	var container;
	var widget;

	var baseContainer = ZhHelper.baseContainerWithTwoWidgets;
	var baseFields = function () {
		return [
			{
				name : "name",
				isVisible: true,
				aclIsVisible: true,
				showOnList: true,
				skipSearch: false
			},
            {
                name: "birth",
                isVisible: true,
                aclIsVisible: true,
                showOnList: true,
                skipSearch: false
            }
		];
	};


	beforeEach(function () {
		module(function ($provide) {
			//Providing a new $timeout because $timeout.flush is not flushing pending $timeout calls
			$provide.constant("$timeout", setTimeout);
			$provide.constant("$q", Q);
		});
		module("ZeedhiApp");
		module("ZeedhiServices");

		inject(function (_$rootScope_, $controller, metaDataFactory) {
			container = baseContainer();
			container.widgets[0].template = "widget/master_detail/grid.html";
			container.widgets[0].templateToForm = "widget/form.html";
			container.widgets[0].templateToView = "widget/form-view.html";
			container.widgets[0].fields = baseFields();
			container = metaDataFactory.containerFactory(container);
			widget = container.widgets[0];
			$rootScope = _$rootScope_;

			$scope = _$rootScope_.$new();
			$scope.$parent = _$rootScope_;
			$scope.widget = widget;

			masterDetailController = $controller("MasterDetailController", {
				$scope: $scope,
				$rootScope: $rootScope
			});
		});
	});

	it("masterDetailController.widget: should return the widget from the scope", function () {
		expect(masterDetailController.widget).toEqual($scope.widget);
	});

	it("$scope.$broadcast('destroy'): the masterDetailController should remove his afterMoveRow and onCloseSwipe listeners", function () {
		var lastAfterMoveRowListenersLength = $scope.widget.afterMoveRowlistener.length;
		var lastOnCloseSwipeListenersLength = $scope.widget.onCloseSwipelistener.length;
		$scope.$broadcast("$destroy");

		expect($scope.widget.afterMoveRowlistener.length).toBe(lastAfterMoveRowListenersLength -1);
		expect($scope.widget.onCloseSwipelistener.length).toBe(lastOnCloseSwipeListenersLength -1);
	});

	it("widget.edit: should set the widget's childTemplate to the widget's templateToForm", function () {
		widget.edit();
		expect(widget.childTemplate).toEqual("widget/form.html");
	});

	it("widget.view: should set the widget's childTemplate to the widget's templateToView", function () {
		widget.view();
		expect(widget.childTemplate).toEqual("widget/form-view.html");
	});

	it("widget.onCloseSwipe: should set the currentWidget as the widget and change to list", function () {
		widget.onCloseSwipe();
		expect(widget.currentWidget).toEqual(widget);
		expect(widget.mode).toEqual(ViewMode.LIST);
	});

});