describe("RequestDataSourceEngine", function() {

	var requestDataSourceEngine;
	var requestEngine;
	var requestFactory;
	var methodBuilder;
	var screenService;
	var zhPromise;
	var zhUtil;
	var dataSourceHolder;
	var templateManager;

	beforeEach(function() {
		requestDataSourceEngine = {};
		requestEngine = {
			doRequest: function() {}
		};
		requestFactory = {
			factory: function() {}
		};
		methodBuilder = {};
		screenService = {};
		zhPromise = new ZHPromise(Q);
		zhUtil = new ZhUtil();
		dataSourceHolder = {
			getDataSource: function() {}
		};
		templateManager = {
			project: {
				timeout: 15000
			}
		};

		requestDataSourceEngine = new RequestDataSourceEngine(requestEngine, requestFactory, methodBuilder, screenService, zhPromise, zhUtil, dataSourceHolder, templateManager);
	});

	it('Should remove prefixes from common filters', function() {
		var filter = [
			{
				"name": "__proxy_HIRE_DATE",
				"operator": ">",
				"value": "2000-01-01 00:00 AM"
			}
		];

		var expectedFilter = [
			{
				"name": "HIRE_DATE",
				"operator": ">",
				"value": "2000-01-01 00:00 AM"
			}
		];

		var expectedRequest = {
			'requestType': 'filterData',
			'filter': expectedFilter,
			'page': 1,
			'itemsPerPage': 500,
			'serviceName': '/employees',
			'callBack': jasmine.any(Function),
			'errorCallBack': jasmine.any(Function),
			'timeout': 15000,
			'retryCount': undefined,
			'retryCallback': jasmine.any(Function)
		};

		spyOn(requestFactory, 'factory').and.callFake(function(request) {
			return request;
		});

		spyOn(requestEngine, 'doRequest');

		requestDataSourceEngine.getDataSource('/employees', filter, 1, 500);

		expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest, '/employees', jasmine.any(Function));
		expect(requestEngine.doRequest).toHaveBeenCalledWith(expectedRequest);
	});

	it('Should remove prefixes from groupBy filters', function() {
		var filter = [
			{
				"name": ["__proxy_HIRE_DATE", "PROXY"],
				"operator": "GROUP_BY",
				"value": null
			}
		];

		var expectedFilter = [
			{
				"name": ["HIRE_DATE", "PROXY"],
				"operator": "GROUP_BY",
				"value": null
			}
		];

		var expectedRequest = {
			'requestType': 'filterData',
			'filter': expectedFilter,
			'page': 1,
			'itemsPerPage': 500,
			'serviceName': '/employees',
			'callBack': jasmine.any(Function),
			'errorCallBack': jasmine.any(Function),
			'timeout': 15000,
			'retryCount': undefined,
			'retryCallback': jasmine.any(Function)
		};

		spyOn(requestFactory, 'factory').and.callFake(function(request) {
			return request;
		});

		spyOn(requestEngine, 'doRequest');

		requestDataSourceEngine.getDataSource('/employees', filter, 1, 500);

		expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest, '/employees', jasmine.any(Function));
		expect(requestEngine.doRequest).toHaveBeenCalledWith(expectedRequest);
	});

	it('Should remove prefixes from orderBy filters', function() {
		var filter = [
			{
				"name": [
					{
						"name": "__proxy_HIRE_DATE",
						"direction": "DESC"
					},
					{
						"name": "ID",
						"direction": "ASC"
					}
				],
				"operator": "GROUP_BY",
				"value": null
			}
		];

		var expectedFilter = [
			{
				"name": [
					{
						"name": "HIRE_DATE",
						"direction": "DESC"
					},
					{
						"name": "ID",
						"direction": "ASC"
					}
				],
				"operator": "GROUP_BY",
				"value": null
			}
		];

		var expectedRequest = {
			'requestType': 'filterData',
			'filter': expectedFilter,
			'page': 1,
			'itemsPerPage': 500,
			'serviceName': '/employees',
			'callBack': jasmine.any(Function),
			'errorCallBack': jasmine.any(Function),
			'timeout': 15000,
			'retryCount': undefined,
			'retryCallback': jasmine.any(Function)
		};

		spyOn(requestFactory, 'factory').and.callFake(function(request) {
			return request;
		});

		spyOn(requestEngine, 'doRequest');

		requestDataSourceEngine.getDataSource('/employees', filter, 1, 500);

		expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest, '/employees', jasmine.any(Function));
		expect(requestEngine.doRequest).toHaveBeenCalledWith(expectedRequest);
	});


});