function AuthController(ScreenService, environment, SocketServiceKDS, ConcludeQueue, templateManager, SoundService, Utilities, SummaryItemsService) {

	var self = this;

	self.PRODUCTION_SECTOR = 'P';
	self.EXPEDITION_SECTOR = 'E';
	self.ASSEMBLY_SECTOR = 'M';

	// variaveis de controle do seletor de setores -> global para poder ser acessada pelo evento
	var fieldOpen = false;
	var focusedElement = -1;
	collection = document.getElementsByClassName("zh-list-item-label"); // pega todos os elementos da lista de setores

	this.loginOnEnter = function(widget){
		var localEnvironment = environment.getLocalEnvironment();
		if (localEnvironment) {
			self.login(localEnvironment, widget);
		} else {
			setBranchOnLoginForm(widget);
			loadSectorsField(widget);
			
			//evento de keydown para escolher o setor
			document.widget = widget; // salva o widget no document para ser acessado no evento
			document.addEventListener("keydown", handleSectorSelection, false);
		}
	};

	function handleSectorSelection(e) {
		var widget = document.widget;

		if (e.key == 'Enter' && fieldOpen == false) { // abre a lista de setores
			fieldOpen = true;
			focusedElement = -1;
			widget.getField('SELECTCDSETOR').openField();
		}
		else if (e.key == 'Enter' && fieldOpen == true) { // seleciona setor
			fieldOpen = false;
			if (collection[focusedElement]) {
				collection[focusedElement].style.fontWeight = "normal";
				// a colletion é uma lista de setores mas o elemento clicável é uma div dentro de cada item da lista, ou seja, o neto do elemento que tem a classe zh-list-item-label
				childrenDiv = collection[focusedElement].children[0]; 
				clickableDiv = childrenDiv.children[0];
				clickableDiv.click();
				focusedElement = -1; // reseta o foco
				setTimeout(() => {
					self.loginButtonOnClick(widget);
				}, 200); // entra no setor
			}
		}
 
		if (e.key == '2' || e.key == 'ArrowDown') { // navega para baixo na lista de setores
			if (focusedElement != -1) collection[focusedElement].style.fontWeight = "normal";
			focusedElement++;
			if (focusedElement > collection.length - 1) { // se for o ultimo elemento, volta para o primeiro
				focusedElement = 0;
			}
			if (collection[focusedElement]) {
				collection[focusedElement].style.fontWeight = "bold";
				collection[focusedElement].focus();
				collection[focusedElement].scrollIntoViewIfNeeded();
			}
		}

		if (e.key == '8' || e.key == 'ArrowUp') { // navega para cima na lista de setores
			if (focusedElement != -1)  collection[focusedElement].style.fontWeight = "normal";
			focusedElement--;
			if (focusedElement < 0) { // se for o primeiro elemento, volta para o ultimo
				focusedElement = collection.length - 1;
			}
			if (collection[focusedElement]) {
				collection[focusedElement].style.fontWeight = "bold";
				collection[focusedElement].focus();
				collection[focusedElement].scrollIntoViewIfNeeded();
			}
		}
	}

	function setBranchOnLoginForm(widget) {
		environment.getBranch().then(function(response) {
			if(response.error) {
				ScreenService.showMessage(response.message, 'ALERT');
			} else {
				var branch = response.branch;
				var branchLabel = branch.CDFILIAL + ' - ' + branch.NMFILIAL;
				widget.getField('LABELFILIAL').value(branchLabel);
				widget.useDarkMode = response.useDarkMode;
				handleDarkMode(response.useDarkMode, widget);
				localStorage.setItem('bindPopupKeys', response.bindPopupKeys);		
				localStorage.setItem('sortItemsByComboOrder', response.sortItemsByComboOrder);	
			}
		});
	}

	function handleDarkMode (useDarkMode, widget) {
		if (useDarkMode) {
			widget.getField('odhen_kds_horizontal').source = 'images/odhen_kds_horizontal_branco.png';

			// salva o parametro de dark mode no local storage para ser utilizado posteriormente
			localStorage.setItem('useDarkMode', true);
			
			document.querySelectorAll(".zh-application-content")[0].classList.add("dark-mode");
		}
	}

	function loadSectorsField(widget) {
		environment.getBranchSectors().then(function(response) {
			if(response.error) {
				ScreenService.showMessage(response.message, 'ALERT');
			} else {
				widget.getField('SELECTCDSETOR').dataSource.data = response.sectors;
			}
		});
	}

	this.loginButtonOnClick = function(widget) {
		document.removeEventListener("keydown", handleSectorSelection, false); // remove o evento de keydown para que não seja chamado quando entrar no setor
		if(widget.isValid()) {
			this.login(widget.currentRow);
		}
	};

	function cleanRequestsOnLogin(){
		return ScreenService.confirmMessage('Você deseja limpar todos os pedidos antes de ir para tela de pedidos?',
		'question',
			function() {
				let localEnvironment = environment.getLocalEnvironment();
				SocketServiceKDS.clearScreen(localEnvironment.CDFILIAL, localEnvironment.CDSETOR, 3);
			},
			function() {}
		);
	}

	this.login = function(row, widget){
		return environment.getRegisterControlParams(row.CDSETOR).then(function(response) {
			if (response.error) {
				ScreenService.showMessage(response.message, 'ALERT');
				environment.unsetLocalEnvironment();
				setBranchOnLoginForm(widget);
				loadSectorsField(widget);
			} else {
				var controlParams = response.productionControlParams;
                
                // Esse parâmetro recebido do backend precisa ser passado para a widget, que ainda não está acessível
                // neste ponto, por isso o localStorage é usado para recuperá-la no momento de incluir na widget
                localStorage.setItem('showDateTime', controlParams.showDateTime);

				controlParams = prepareControlParams(row, controlParams);
				environment.setLocalEnvironment(controlParams);
				SoundService.configureBeep();
				environment.setProjectConfig().then((data)=>{
					SocketServiceKDS.startWebSocket();
					if(data.cleanRequestsOnLogin){
						cleanRequestsOnLogin().then(()=> {
							redirectToCorrectSector(controlParams.IDTIPOSETOR);
						}, ()=>{
							redirectToCorrectSector(controlParams.IDTIPOSETOR);
						});
					} else {
						redirectAndStartSocket(controlParams.IDTIPOSETOR);
					}
				});

			}
		});
	};

	function prepareControlParams (row, controlParams) {
		return {
			'CDSETOR'          : row.CDSETOR,
			'IDTIPOSETOR'      : row.IDTIPOSETOR,
			'CDFILIAL'	       : controlParams.CDFILIAL,
			'labelContainer'   : 'KDS - Setor: '+ row.NMSETOR,
			'NMSETOR'          : row.NMSETOR,
			'hasSummary'       : row.hasSummary == undefined && row.IDTIPOSETOR == 'P' ? true : row.hasSummary,
			'IDHABROLLBKDS'    : controlParams.IDHABROLLBKDS,
			'IDHABSOMKDS'      : controlParams.IDHABSOMKDS,
			'IDHABTIMERPROD'   : controlParams.IDHABTIMERPROD,
			'IDHABSOMPROD'     : controlParams.IDHABSOMPROD,
			'IDHABCONTSOMPROD' : controlParams.IDHABCONTSOMPROD,
			'IDBLOQPRODKDS'    : controlParams.IDBLOQPRODKDS,
			'NRROLLBTIME'      : controlParams.NRROLLBTIME === null ? 0 : controlParams.NRROLLBTIME * 1000,
			'IDCONTSOMKDS'     : controlParams.IDCONTSOMKDS,
			'IDHABPREENVIO'    : controlParams.IDHABPREENVIO === 'S',
			'NRATRASOSOMPROD'  : controlParams.NRATRASOSOMPROD,
			'NRPRODATRAINISOM' : controlParams.NRPRODATRAINISOM,
			'PLAYANDCONCLUDE'  : controlParams.IDCONCDIRETO === 'S',
			'SOMBEEP'          : controlParams.SOMBEEP,
			'keyMapping'	   : controlParams.keyMapping,
			'concludeDisabled' : controlParams.IDBLOQPRODKDS === 'S',
			'IDMOSTRAORIGEM'   : controlParams.IDMOSTRAORIGEM === 'S',
			'NRTEMPOATRASOEXP' : controlParams.NRTEMPOATRASOEXP,
			'SOMBEEPEXP'       : controlParams.SOMBEEPEXP,
			'NRMULTIPULO'      : controlParams.NRMULTIPULO,
            'IDHABAGRUPPROD'   : controlParams.IDHABAGRUPPROD,
			'IDLIBCONDPG'      : controlParams.IDLIBCONDPG,
			'IDFRNONNTF'	   : controlParams.IDFRNONNTF,
			'IDCONCAUT'		   : controlParams.IDCONCAUT,
			'IDHABCANCDLV'     : controlParams.IDHABCANCDLV,
			'IDHABCANCBAL'     : controlParams.IDHABCANCBAL,
			'IDHABCANCMES'     : controlParams.IDHABCANCMES,
			'IDHABCANCCMD'     : controlParams.IDHABCANCCMD,
			'IDHABCANCTHR'     : controlParams.IDHABCANCTHR,
			'IDHABCANCTGO'     : controlParams.IDHABCANCTGO,
			'IDAUMENTARESUMO'  : controlParams.IDAUMENTARESUMO,
			'IDHABESTATISTICAS': controlParams.IDHABESTATISTICAS,
			'IDHABSCANCODE'	   : controlParams.IDHABSCANCODE,
			'IDHABLIBDIRETOEXP': controlParams.IDHABLIBDIRETOEXP,
			'IDHABESTGERAL'	   : controlParams.IDHABESTGERAL,
            'IDHABEXPFLUID'	   : controlParams.IDHABEXPFLUID,
			'IDEXIBCONDLIB'    : controlParams.IDEXIBCONDLIB,
			'CDLINHAPROD'      : controlParams.CDLINHAPROD,
			'IDLINHAPRINCIPAL' : controlParams.IDLINHAPRINCIPAL,
			'IDHABAGRUPPRODPOSMESA' : controlParams.IDHABAGRUPPRODPOSMESA,
			'IDHABPRIORIDADE'  : controlParams.IDHABPRIORIDADE,
			'IDORDPEDCON': controlParams.IDORDPEDCON
		};
	}

	function redirectToCorrectSector(IDTIPOSETOR) {
		switch(IDTIPOSETOR){
			case self.PRODUCTION_SECTOR:
				// ScreenService.openWindow('ordenation');
				return ScreenService.openWindow('production');
				
			case self.EXPEDITION_SECTOR:
			case self.ASSEMBLY_SECTOR:
				// ScreenService.openWindow('ordenation');
				return ScreenService.openWindow('expedition');
		}
	}

	function redirectAndStartSocket(IDTIPOSETOR) {
		redirectToCorrectSector(IDTIPOSETOR);
	}

	this.logoff = function() {
		ScreenService.toggleSideMenu();
		if(ConcludeQueue.isQueueEmpty()) {
			logoffConfirmation();
		} else {
			openLastOpenedWindow();
			ScreenService.showMessage('Ainda existem pedidos aguardando para serem liberados, favor aguardar');
		}
	};

	function logoffConfirmation() {
		clearKDSWidgetDataSource();
		SocketServiceKDS.logoffBrowser();
		SummaryItemsService.setFixedItems(null);
		environment.unsetLocalEnvironment();
		Utilities.clearAllTimeOutsAndIntervals();
		ScreenService.openWindow('initialParameters');
	}

	function openLastOpenedWindow() {
		var lastOpenedContainerName = templateManager.historyItems[templateManager.historyItems.length - 2];
		ScreenService.openWindow(lastOpenedContainerName);
	}

	function clearKDSWidgetDataSource() {
		var widget =  environment.getKDSWidget();
		widget.dataSource.data = [];
	}

}

Configuration(function(ContextRegister) {
	ContextRegister.register('AuthController', AuthController);
});